/* Notice to developers: this file is intentionally included twice. */

/** \file
    \brief Gen3 Electronics Sample Configuration
*/

/*
    CONTENTS

    1. Mechanical/Hardware
    2. Acceleration settings
    3. Pinouts
    4. Temperature sensors
    5. Heaters
    6. Communication options
    7. Miscellaneous
    8. Appendix A - PWMable pins and mappings
*/

/***************************************************************************\
*                                                                           *
* 1. MECHANICAL/HARDWARE                                                    *
*                                                                           *
\***************************************************************************/

/*
    Set your microcontroller type in Makefile! atmega168/atmega328p/atmega644p/atmega1280

    If you want to port this to a new chip, start off with arduino.h and see how you go.
*/
#if ! ( defined (__AVR_ATmega644P__) || defined (__AVR_ATmega644PA__) )
    #error GEN3 has a 644P/644PA! set your cpu type in Makefile!
#endif

/** \def F_CPU
    CPU clock rate
*/
#ifndef F_CPU
    #define F_CPU   16000000UL
#endif

/** \def MOTHERBOARD
    This is the motherboard, as opposed to the extruder. See extruder/ directory for GEN3 extruder firmware
*/
#define MOTHERBOARD


/** \def STEPS_PER_M
    steps per meter ( = steps per mm * 1000 )

    calculate these values appropriate for your machine

    for threaded rods, this is
        (steps motor per turn) / (pitch of the thread) * 1000

    for belts, this is
        (steps per motor turn) / (number of gear teeth) / (belt module) * 1000

    half-stepping doubles the number, quarter stepping requires * 4, etc.

    valid range = 20 to 4'0960'000 (0.02 to 40960 steps/mm)

    all numbers are integers, so no decimal point, please :-)
*/
//#define   STEPS_PER_M_X                   10047
//#define   STEPS_PER_M_X                   8000
//rechnerisch 8000, experimentell auf der X-Achse 8963.59
//rechnerisch 8000, experimentell auf der X-Achse 8075.67
#define STEPS_PER_M_X                   8200
// 200 steps / 20 gearteeth / 2.5mm (abtsand zahnriemen-zähne)
//#define   STEPS_PER_M_Y                   10047
//#define   STEPS_PER_M_Y                   8000
//rechnerisch 8000, experimentell auf der Y-Achse 8164.0
#define STEPS_PER_M_Y                   7950
//160000 für single step mit M8 Muttern, 320000 für half-step mit M8 Muttern
//200 steps / 1.25 * 1000 * 2
#define STEPS_PER_M_Z                   320000

/// http://blog.arcol.hu/?p=157 may help with this one
//#define   STEPS_PER_M_E                   35200
// Extruder gear ratio Gr = 14.0 / 43.0
// Extruder nozzle diameter = 0.5mm (bzw 0.35mm mit unserer anderen Düse)
// Filament Width = 3mm


// d_schraube ~= 0.007m
// u_schraube = d_schraube * pi ~= 0.022m
// motor_steps for 1 circumfence = 200
// Extruder gear ratio Gr = 14.0 / 43.0
// STEPS_PER_M_E = 1m / (Gr * u_schraube) * motor_steps ¨~= 27933
//#define   STEPS_PER_M_E       27933
//experimentell (ohne Düse): 99455 previousvalue *(200.0mm software/194.0mm echt) = 102530.93
#define STEPS_PER_M_E   102531


/*
    Values depending on the capabilities of your stepper motors and other mechanics.
        All numbers are integers, no decimals allowed.

        Units are mm/min
*/

/// used for G0 rapid moves and as a cap for all other feedrates
#define MAXIMUM_FEEDRATE_X      5000
#define MAXIMUM_FEEDRATE_Y      5000
#define MAXIMUM_FEEDRATE_Z      100
#define MAXIMUM_FEEDRATE_E      200

/// used when searching endstops and as default feedrate
#define SEARCH_FEEDRATE_X           50
#define SEARCH_FEEDRATE_Y           50
#define SEARCH_FEEDRATE_Z           50
// no SEARCH_FEEDRATE_E, as E can't be searched

/** \def SLOW_HOMING
    wether to search the home point slowly
        With some endstop configurations, like when probing for the surface of a PCB, you can't deal with overrunning the endstop. In such a case, uncomment this definition.
*/
// #define  SLOW_HOMING

/// this is how many steps to suck back the filament by when we stop. set to zero to disable
#define E_STARTSTOP_STEPS           20

/**
    Soft axis limits, in mm.
    Define them to your machine's size relative to what your host considers to be the origin.
*/

#define X_MIN           0.0
// eigentlich 184mm, aber wir lassen uns ein bisschen Sicherheitsabstand
//#define   X_MAX           184.0
#define X_MAX           177.0

#define Y_MIN           0.0
#define Y_MAX           195.0

#define Z_MIN           0.0
#define Z_MAX           125.0

/** \def E_ABSOLUTE
    Some G-Code creators produce relative length commands for the extruder, others absolute ones. G-Code using absolute lengths can be recognized when there are G92 E0 commands from time to time. If you have G92 E0 in your G-Code, define this flag.

    This is the startup default and can be changed with M82/M83 while running.
*/
#define E_ABSOLUTE



/***************************************************************************\
*                                                                           *
* 2. ACCELERATION                                                           *
*                                                                           *
* IMPORTANT: choose only one! These algorithms choose when to step, trying  *
*            to use more than one will have undefined and probably          *
*            disastrous results!                                            *
*                                                                           *
\***************************************************************************/

/** \def ACCELERATION_REPRAP
    acceleration, reprap style.
        Each movement starts at the speed of the previous command and accelerates or decelerates linearly to reach target speed at the end of the movement.
*/
// #define ACCELERATION_REPRAP


/** \def ACCELERATION_RAMPING
    acceleration and deceleration ramping.
        Each movement starts at (almost) no speed, linearly accelerates to target speed and decelerates just in time to smoothly stop at the target. alternative to ACCELERATION_REPRAP
*/
#define ACCELERATION_RAMPING

/** \def ACCELERATION
    how fast to accelerate when using ACCELERATION_RAMPING.
        given in mm/s^2, decimal allowed, useful range 1. to 10'000. Start with 10. for milling (high precision) or 1000. for printing
*/
#define ACCELERATION 1000.

/** \def ACCELERATION_TEMPORAL
    temporal step algorithm
        This algorithm causes the timer to fire when any axis needs to step, instead of synchronising to the axis with the most steps ala bresenham.

        This algorithm is not a type of acceleration, and I haven't worked out how to integrate acceleration with it.
        However it does control step timing, so acceleration algorithms seemed appropriate

        The Bresenham algorithm is great for drawing lines, but not so good for steppers - In the case where X steps 3 times to Y's two, Y experiences massive jitter as it steps in sync with X every 2 out of 3 X steps. This is a worst-case, but the problem exists for most non-45/90 degree moves. At higher speeds, the jitter /will/ cause position loss and unnecessary vibration.
        This algorithm instead calculates when a step occurs on any axis, and sets the timer to that value.

        \todo figure out how to add acceleration to this algorithm
*/
// #define ACCELERATION_TEMPORAL



/***************************************************************************\
*                                                                           *
* 3. PINOUTS                                                                *
*                                                                           *
\***************************************************************************/

/**
    Machine Pin Definitions
    - make sure to avoid duplicate usage of a pin
    - comment out pins not in use, as this drops the corresponding code and makes operations faster
*/

#include    "arduino.h"

/** \def USE_INTERNAL_PULLUPS
    internal pullup resistors
        the ATmega has internal pullup resistors on it's input pins which are counterproductive with the commonly used eletronic endstops, so they should be switched off. For other endstops, like mechanical ones, you may want to uncomment this.
*/
//#define USE_INTERNAL_PULLUPS

/**
    this is the official gen3 reprap motherboard pinout
*/
#define TX_ENABLE_PIN                   DIO12
#define RX_ENABLE_PIN                   DIO13

#define X_STEP_PIN                      DIO15
#define X_DIR_PIN                           DIO18
//~ #define X_MIN_PIN                           DIO20
#define X_MAX_PIN                           DIO21
#define X_ENABLE_PIN                    DIO19
//#define   X_INVERT_DIR
//~ #define X_INVERT_MIN
//~ #define X_INVERT_MAX
#define X_INVERT_ENABLE

#define Y_STEP_PIN                      DIO23
#define Y_DIR_PIN                           DIO22
#define Y_MIN_PIN                           AIO6
//~ #define Y_MAX_PIN                           AIO5
#define Y_ENABLE_PIN                    AIO7
//#define   Y_INVERT_DIR
//~ #define Y_INVERT_MIN
//~ #define Y_INVERT_MAX
#define Y_INVERT_ENABLE

#define Z_STEP_PIN                      AIO4
#define Z_DIR_PIN                           AIO3
#define Z_MIN_PIN                           AIO1
//~ #define Z_MAX_PIN                           AIO0
#define Z_ENABLE_PIN                    AIO2
//#define   Z_INVERT_DIR
//~ #define Z_INVERT_MIN
//~ #define Z_INVERT_MAX
#define Z_INVERT_ENABLE

#define E_STEP_PIN                      DIO17
#define E_DIR_PIN                           DIO16
//#define E_ENABLE_PIN                  xxxx
//#define   E_INVERT_DIR
//#define   E_INVERT_ENABLE

#define SD_CARD_DETECT              DIO2
#define SD_WRITE_PROTECT            DIO3

//#define   PS_ON_PIN                           DIO14
#define PS_ON_PIN                           PB5 // on TechZoneComm Motherboard, SPI Port: MOSI
//#define   STEPPER_ENABLE_PIN      xxxx
//#define   STEPPER_INVERT_ENABLE



/***************************************************************************\
*                                                                           *
* 4. TEMPERATURE SENSORS                                                    *
*                                                                           *
\***************************************************************************/

/**
    TEMP_HYSTERESIS: actual temperature must be target +/- hysteresis before target temperature can be achieved.
    Unit is degree Celsius.
*/
#define TEMP_HYSTERESIS             5

/**
    TEMP_RESIDENCY_TIME: actual temperature must be close to target (within
    set temperature +- TEMP_HYSTERESIS) for this long before target is achieved
    (and a M116 succeeds). Unit is seconds.
*/
#define TEMP_RESIDENCY_TIME     30

/**
  TEMP_EWMA: Smooth noisy temperature sensors. Good hardware shouldn't be
  noisy. Set to 1.0 for unfiltered data (and a 140 bytes smaller binary).

  Instrument Engineer's Handbook, 4th ed, Vol 2 p126 says values of
  0.05 to 0.1 are typical. Smaller is smoother but slower adjusting, larger is
  quicker but rougher. If you need to use this, set the PID parameter to zero
  (M132 S0) to make the PID loop insensitive to noise.

  Valid range: 0.001 to 1.0
*/
#define TEMP_EWMA             0.1

/// which temperature sensors are you using? List every type of sensor you use here once, to enable the appropriate code. Intercom is the gen3-style separate extruder board.
// #define  TEMP_MAX6675
// #define  TEMP_THERMISTOR
// #define  TEMP_AD595
// #define  TEMP_PT100
#define TEMP_INTERCOM

/***************************************************************************\
*                                                                           *
* Define your temperature sensors here. One line for each sensor, only      *
* limited by the number of available ATmega pins.                           *
*                                                                           *
* Types are same as TEMP_ list above - TT_MAX6675, TT_THERMISTOR, TT_AD595, *
*   TT_PT100, TT_INTERCOM. See list in temp.c.                              *
*                                                                           *
* The "additional" field is used for TT_THERMISTOR only. It defines the     *
* name of the table(s) in ThermistorTable.h to use. Typically, this is      *
* THERMISTOR_EXTRUDER for the first or only table, or THERMISTOR_BED for    *
* the second table. See also early in ThermistorTable.{single|double}.h.    *
*                                                                           *
* For a GEN3 set temp_type to TT_INTERCOM and temp_pin to AIO0. The pin     *
* won't be used in this case.                                               *
*                                                                           *
\***************************************************************************/

#ifndef DEFINE_TEMP_SENSOR
    #define DEFINE_TEMP_SENSOR(...)
#endif

//                 name       type            pin        additional
DEFINE_TEMP_SENSOR(noheater,  TT_INTERCOM,    AIO0,      0)
DEFINE_TEMP_SENSOR(bed,       TT_INTERCOM,    AIO1,      0)

// bed has no heater attached
#define HEATER_bed HEATER_noheater


/***************************************************************************\
*                                                                           *
* 5. HEATERS                                                                *
*                                                                           *
\***************************************************************************/

/** \def HEATER_SANITY_CHECK
    check if heater responds to changes in target temperature, disable and spit errors if not
    largely untested, please comment in forum if this works, or doesn't work for you!
*/
// #define  HEATER_SANITY_CHECK

/***************************************************************************\
*                                                                           *
* Define your heaters and devices here.                                     *
*                                                                           *
* To attach a heater to a temp sensor above, simply use exactly the same    *
* name - copy+paste is your friend. Some common names are 'extruder',       *
* 'bed', 'fan', 'motor', ... names with special meaning can be found        *
* in gcode_process.c. Currently, these are:                                 *
*   HEATER_extruder   (M104)                                                *
*   HEATER_bed        (M140)                                                *
*   HEATER_fan        (M106)                                                *
*                                                                           *
* Devices don't neccessarily have a temperature sensor, e.g. fans or        *
* milling spindles. Operate such devices by setting their power (M106),     *
* instead of setting their temperature (M104).                              *
*                                                                           *
* Also note, the index of a heater (M106 P#) can differ from the index of   *
* its attached temperature sensor (M104 P#) in case sensor-less devices     *
* are defined or the order of the definitions differs. The first defined    *
* device has the index 0 (zero).                                            *
*                                                                           *
* Set 'pwm' to ...                                                          *
*  1  for using PWM on a PWM-able pin and on/off on other pins.             *
*  0  for using on/off on a PWM-able pin, too.                              *
* Using PWM usually gives smoother temperature control but can conflict     *
* with slow switches, like solid state relays. PWM frequency can be         *
* influenced globally with FAST_PWM, see below.                             *
*                                                                           *
\***************************************************************************/

#ifndef DEFINE_HEATER
    #define DEFINE_HEATER(...)
#endif

//            name      port   pwm
// DEFINE_HEATER(extruder, PINB3, 1)
// DEFINE_HEATER(bed,      PINB4, 1)

/** \def HEATER_EXTRUDER
    \def HEATER_BED
    \def HEATER_FAN
    and now because the c preprocessor isn't as smart as it could be,
    uncomment the ones you've listed above and comment the rest.
    \NOTE these are used to enable various capability-specific chunks of code, you do NOT need to create new entries unless you are adding new capabilities elsewhere in the code!
    so if you list a bed above, uncomment HEATER_BED, but if you list a chamber you do NOT need to create HEATED_CHAMBER
    I have searched high and low for a way to make the preprocessor do this for us, but so far I have not found a way.
*/

// workaround for heated bed over intercom. HEATER_EXTRUDER, HEATER_BED are the intercom temp channel numbers.
#define HEATER_EXTRUDER 0
#define HEATER_BED 1
#define HEATER_FAN 2



/***************************************************************************\
*                                                                           *
* 6. COMMUNICATION OPTIONS                                                  *
*                                                                           *
\***************************************************************************/

/** \def BAUD
  Baud rate for the serial RS232 protocol connection to the host. Usually
  115200, other common values are 19200, 38400 or 57600. Ignored when USB_SERIAL
  is defined.
*/
#define BAUD 115200

/** \def USB_SERIAL
  Define this for using USB instead of the serial RS232 protocol. Works on
  USB-equipped ATmegas, like the ATmega32U4, only.
*/
// #define USB_SERIAL

/** \def XONXOFF
    Xon/Xoff flow control.
        Redundant when using RepRap Host for sending GCode, but mandatory when sending GCode files with a plain terminal emulator, like GtkTerm (Linux), CoolTerm (Mac) or HyperTerminal (Windows).
        Can also be set in Makefile
*/
// #define  XONXOFF



/***************************************************************************\
*                                                                           *
* 7. MISCELLANEOUS OPTIONS                                                  *
*                                                                           *
\***************************************************************************/

/** \def EECONFIG
  EECONFIG: Enable EEPROM configuration storage.

  Enabled by default. Commenting this out makes the binary several hundred
  bytes smaller, so you might want to disable EEPROM storage on small MCUs,
  like the ATmega168.
*/
#define EECONFIG

/** \def DEBUG
    DEBUG
        enables /heaps/ of extra output, and some extra M-codes.
        WARNING: this WILL break most host-side talkers that expect particular responses from firmware such as reprap host and replicatorG
        use with serial terminal or other suitable talker only.
*/
// #define  DEBUG

/** \def BANG_BANG
    BANG_BANG
        drops PID loop from heater control, reduces code size significantly (1300 bytes!)
        may allow DEBUG on '168
*//** \def BANG_BANG_ON
    BANG_BANG_ON
        PWM value for 'on'
*//** \def BANG_BANG_OFF
    BANG_BANG_OFF
        PWM value for 'off'
*/
// #define  BANG_BANG
// #define  BANG_BANG_ON    200
// #define  BANG_BANG_OFF   45

/**
    move buffer size, in number of moves
        note that each move takes a fair chunk of ram (69 bytes as of this writing) so don't make the buffer too big - a bigger serial readbuffer may help more than increasing this unless your gcodes are more than 70 characters long on average.
        however, a larger movebuffer will probably help with lots of short consecutive moves, as each move takes a bunch of math (hence time) to set up so a longer buffer allows more of the math to be done during preceding longer moves
*/
#define MOVEBUFFER_SIZE 8

/** \def DC_EXTRUDER
    DC extruder
        If you have a DC motor extruder, configure it as a "heater" above and define this value as the index or name. You probably also want to comment out E_STEP_PIN and E_DIR_PIN in the Pinouts section above.
*/
// #define  DC_EXTRUDER HEATER_motor
// #define  DC_EXTRUDER_PWM 180

/** \def USE_WATCHDOG
    Teacup implements a watchdog, which has to be reset every 250ms or it will reboot the controller. As rebooting (and letting the GCode sending application trying to continue the build with a then different Home point) is probably even worse than just hanging, and there is no better restore code in place, this is disabled for now.
*/
// #define USE_WATCHDOG

/**
    analog subsystem stuff
    REFERENCE - which analog reference to use. see analog.h for choices
*/
#define REFERENCE           REFERENCE_AVCC

/** \def STEP_INTERRUPT_INTERRUPTIBLE
    this option makes the step interrupt interruptible (nested).
    this should help immensely with dropped serial characters, but may also make debugging infuriating due to the complexities arising from nested interrupts
*/
#define     STEP_INTERRUPT_INTERRUPTIBLE    1

/**
    temperature history count. This is how many temperature readings to keep in order to calculate derivative in PID loop
    higher values make PID derivative term more stable at the expense of reaction time
*/
#define TH_COUNT                    8

/** \def FAST_PWM
    Teacup offers two PWM frequencies, 76(61) Hz and 78000(62500) Hz on a
    20(16) MHz electronics. The slower one is the default, as it's the safer
    choice. Drawback is, in a quiet environment you might notice the heaters
    and your power supply humming.

    Uncomment this option if you want to get rid of this humming or want
    faster PWM for other reasons.

    See also: http://reprap.org/wiki/Gen7_Research#MOSFET_heat_and_PWM
*/
// #define  FAST_PWM

/// this is the scaling of internally stored PID values. 1024L is a good value
#define PID_SCALE                       1024L

/** \def ENDSTOP_STEPS
    number of steps to run into the endstops intentionally
        As Endstops trigger false alarm sometimes, Teacup debounces them by counting a number of consecutive positives. Valid range is 1...255. Use 4 or less for reliable endstops, 8 or even more for flaky ones.
*/
#define ENDSTOP_STEPS   4



/***************************************************************************\
*                                                                           *
* 8. APPENDIX A - PWMABLE PINS AND MAPPINGS                                 *
*                                                                           *
*                                                                           *
* list of PWM-able pins and corresponding timers                            *
* timer1 is used for step timing so don't use OC1A/OC1B                     *
* they are omitted from this listing for that reason                        *
*                                                                           *
* For the atmega168/328, timer/pin mappings are as follows                  *
*                                                                           *
* OCR0A - PD6  - DIO6                                                       *
* OCR0B - PD5  - DIO5                                                       *
* OCR2A - PB3  - DIO11                                                      *
* OCR2B - PD3  - DIO3                                                       *
*                                                                           *
* For the atmega644, timer/pin mappings are as follows                      *
*                                                                           *
* OCR0A - PB3  - DIO3                                                       *
* OCR0B - PB4  - DIO4                                                       *
* OCR2A - PD7  - DIO15                                                      *
* OCR2B - PD6  - DIO14                                                      *
*                                                                           *
* For the atmega1280, timer/pin mappings are as follows                     *
*                                                                           *
* OCR0A  - PB7 - DIO13                                                      *
* OCR0B  - PG5 - DIO4                                                       *
* OCR2A  - PB4 - DIO10                                                      *
* OCR2B  - PH6 - DIO9                                                       *
* OCR3AL - PE3 - DIO5                                                       *
* OCR3BL - PE4 - DIO2                                                       *
* OCR3CL - PE5 - DIO3                                                       *
* OCR4AL - PH3 - DIO6                                                       *
* OCR4BL - PH4 - DIO7                                                       *
* OCR4CL - PH5 - DIO8                                                       *
* OCR5AL - PL3 - DIO46                                                      *
* OCR5BL - PL4 - DIO45                                                      *
* OCR5CL - PL5 - DIO44                                                      *
*                                                                           *
\***************************************************************************/
